Index: encoder/encoder.c
===================================================================
--- encoder/encoder.c	(revision 680)
+++ encoder/encoder.c	(working copy)
@@ -113,7 +113,7 @@
     sh->pps = pps;
 
     sh->i_first_mb  = 0;
-    sh->i_last_mb   = h->sps->i_mb_width * h->sps->i_mb_height;
+    sh->i_last_mb   = (param->i_slice_size_mb_threshold == 0) ? h->sps->i_mb_width * h->sps->i_mb_height - 1 : param->i_slice_size_mb_threshold - 1;
     sh->i_pps_id    = pps->i_id;
 
     sh->i_frame_num = i_frame;
@@ -698,6 +698,8 @@
             *h->thread[i] = *h;
         h->thread[i]->fdec = x264_frame_pop_unused( h );
         h->thread[i]->out.p_bitstream = x264_malloc( h->out.i_bitstream );
+        h->thread[i]->out.nal = x264_malloc( sizeof ( x264_nal_t) * X264_INIT_NAL_NUM);
+        h->thread[i]->out.i_nals_allocated = X264_INIT_NAL_NUM;
         if( x264_macroblock_cache_init( h->thread[i] ) < 0 )
             return NULL;
     }
@@ -770,6 +772,7 @@
     nal->i_ref_idc = i_ref_idc;
     nal->i_type    = i_type;
 
+    nal->i_numb_mbs= 0;
     nal->i_payload= 0;
     nal->p_payload= &h->out.p_bitstream[bs_pos( &h->out.bs ) / 8];
 }
@@ -778,6 +781,12 @@
     x264_nal_t *nal = &h->out.nal[h->out.i_nal];
     nal->i_payload = &h->out.p_bitstream[bs_pos( &h->out.bs ) / 8] - nal->p_payload;
     h->out.i_nal++;
+    //if number of allocated nals is not enough, re-allocate a larger one (steps of 1) 
+    if(h->out.i_nal >= h->out.i_nals_allocated)
+    {
+        h->out.nal = x264_realloc(h->out.nal, sizeof ( x264_nal_t) * (h->out.i_nals_allocated + 1));
+        h->out.i_nals_allocated++;
+    }
 }
 
 /****************************************************************************
@@ -1021,10 +1030,8 @@
     int i_skip;
     int mb_xy;
     int i;
+    int slice_bits_remain;
 
-    /* init stats */
-    memset( &h->stat.frame, 0, sizeof(h->stat.frame) );
-
     /* Slice */
     x264_nal_start( h, h->i_nal_type, h->i_nal_ref_idc );
 
@@ -1041,8 +1048,11 @@
     }
     h->mb.i_last_qp = h->sh.i_qp;
     h->mb.i_last_dqp = 0;
+    
+    /* slice size threshold in bits */
+    slice_bits_remain = (h->param.slice_size_threshold<<3);
 
-    for( mb_xy = h->sh.i_first_mb, i_skip = 0; mb_xy < h->sh.i_last_mb; )
+    for( mb_xy = h->sh.i_first_mb, i_skip = 0; mb_xy <= h->sh.i_last_mb; )
     {
         const int i_mb_y = mb_xy / h->sps->i_mb_width;
         const int i_mb_x = mb_xy % h->sps->i_mb_width;
@@ -1145,6 +1155,13 @@
         }
         else
             mb_xy++;
+        //end slice if a) finished frame b) there is a max slice size and it has been exceeded
+        slice_bits_remain -= (bs_pos(&h->out.bs) + x264_cabac_pos(&h->cabac) - mb_spos);
+        if ((mb_xy > h->sh.i_last_mb)
+            || ((h->param.slice_size_threshold > 0) && (slice_bits_remain<0))) 
+        {
+           break;
+        }
     }
 
     if( h->param.b_cabac )
@@ -1168,14 +1185,16 @@
         bs_rbsp_trailing( &h->out.bs );
     }
 
+    h->out.nal[h->out.i_nal].i_numb_mbs = mb_xy - h->sh.i_first_mb; 
     x264_nal_end( h );
-
-    /* Compute misc bits */
-    h->stat.frame.i_misc_bits = bs_pos( &h->out.bs )
-                              + NALU_OVERHEAD * 8
-                              - h->stat.frame.i_itex_bits
-                              - h->stat.frame.i_ptex_bits
-                              - h->stat.frame.i_hdr_bits;
+    h->sh.i_first_mb = mb_xy;
+	
+	if( h->param.i_slice_size_mb_threshold > 0 )
+	{
+		int i_frame_size = h->sps->i_mb_width * h->sps->i_mb_height;
+		int i_last_mb = mb_xy + h->param.i_slice_size_mb_threshold - 1;
+		h->sh.i_last_mb = (i_last_mb > i_frame_size - 1) ? i_frame_size - 1 : i_last_mb;
+	}
 }
 
 static void x264_thread_sync_context( x264_t *dst, x264_t *src )
@@ -1207,15 +1226,27 @@
 
 static int x264_slices_write( x264_t *h )
 {
-    int i_frame_size;
+    int i_frame_size = 0;
 
 #if VISUALIZE
     if( h->param.b_visualize )
         x264_visualize_init( h );
 #endif
 
-    x264_stack_align( x264_slice_write, h );
-    i_frame_size = h->out.nal[h->out.i_nal-1].i_payload;
+    /* init stats */
+    memset( &h->stat.frame, 0, sizeof(h->stat.frame) );
+
+    while(h->sh.i_first_mb < h->sh.i_last_mb)
+    {
+        x264_stack_align( x264_slice_write, h );
+        i_frame_size += h->out.nal[h->out.i_nal-1].i_payload;
+    }
+    h->stat.frame.i_misc_bits = bs_pos( &h->out.bs )
+                              + (h->out.i_nal*NALU_OVERHEAD * 8)
+                              - h->stat.frame.i_itex_bits
+                              - h->stat.frame.i_ptex_bits
+                              - h->stat.frame.i_hdr_bits;
+
     x264_fdec_filter_row( h, h->sps->i_mb_height );
 
 #if VISUALIZE
@@ -1643,9 +1674,9 @@
 
     /* Slice stat */
     h->stat.i_slice_count[h->sh.i_type]++;
-    h->stat.i_slice_size[h->sh.i_type] += h->out.i_frame_size + NALU_OVERHEAD;
+    h->stat.i_slice_size[h->sh.i_type] += (h->out.i_frame_size + (h->out.i_nal*NALU_OVERHEAD));
     h->stat.i_slice_qp[h->sh.i_type] += h->fdec->i_qpplus1 - 1;
-
+   
     for( i = 0; i < X264_MBTYPE_MAX; i++ )
         h->stat.i_mb_count[h->sh.i_type][i] += h->stat.frame.i_mb_count[i];
     for( i = 0; i < 2; i++ )
@@ -1960,6 +1991,7 @@
     {
         x264_macroblock_cache_end( h->thread[i] );
         x264_free( h->thread[i]->out.p_bitstream );
+        x264_free( h->thread[i]->out.nal);
         x264_free( h->thread[i] );
     }
 }
Index: encoder/analyse.c
===================================================================
--- encoder/analyse.c	(revision 680)
+++ encoder/analyse.c	(working copy)
@@ -227,10 +227,25 @@
         int i_fpel_border = 5; // umh unconditional radius
         int i_spel_border = 8; // 1.5 for subpel_satd, 1.5 for subpel_rd, 2 for bime, round up
 
+		/* Calculate first_mb and last_mb coordinates */
+		int i_first_mb_x = h->sh.i_first_mb % h->sps->i_mb_width;
+		int i_first_mb_y = ( h->sh.i_first_mb - i_first_mb_x ) / h->sps->i_mb_width;
+		int i_last_mb_x = h->sh.i_last_mb % h->sps->i_mb_width;
+		int i_last_mb_y = ( h->sh.i_last_mb - i_last_mb_x ) / h->sps->i_mb_width;
+
         /* Calculate max allowed MV range */
 #define CLIP_FMV(mv) x264_clip3( mv, -i_fmv_range, i_fmv_range )
-        h->mb.mv_min[0] = 4*( -16*h->mb.i_mb_x - 24 );
-        h->mb.mv_max[0] = 4*( 16*( h->sps->i_mb_width - h->mb.i_mb_x - 1 ) + 24 );
+		/* Special case if the option --no-mv-inter-slices */
+		if( h->param.b_no_mv_inter_slices == 0 )
+		{
+			h->mb.mv_min[0] = 4*( -16*h->mb.i_mb_x - 24 );
+			h->mb.mv_max[0] = 4*( 16*( h->sps->i_mb_width - h->mb.i_mb_x - 1 ) + 24 );
+		}
+		else
+		{
+			h->mb.mv_min[0] = ( i_first_mb_y == h->mb.i_mb_y ) ? 4*( -16*( h->mb.i_mb_x - i_first_mb_x ) - 8 ) : 4*( -16*h->mb.i_mb_x - 8 );
+			h->mb.mv_max[0] = ( i_last_mb_y == h->mb.i_mb_y ) ? 4*( 16*( i_last_mb_x - h->mb.i_mb_x - 1 ) + 8 ) : 4*( 16*( h->sps->i_mb_width - h->mb.i_mb_x - 1 ) + 8 );
+		}
         h->mb.mv_min_spel[0] = CLIP_FMV( h->mb.mv_min[0] );
         h->mb.mv_max_spel[0] = CLIP_FMV( h->mb.mv_max[0] );
         h->mb.mv_min_fpel[0] = (h->mb.mv_min_spel[0]>>2) + i_fpel_border;
@@ -239,6 +254,7 @@
         {
             int mb_y = h->mb.i_mb_y >> h->sh.b_mbaff;
             int mb_height = h->sps->i_mb_height >> h->sh.b_mbaff;
+			int last_mb_y = i_last_mb_y >> h->sh.b_mbaff;
             int thread_mvy_range = i_fmv_range;
 
             if( h->param.i_threads > 1 )
@@ -261,8 +277,16 @@
                     thread_mvy_range >>= 1;
             }
 
-            h->mb.mv_min[1] = 4*( -16*mb_y - 24 );
-            h->mb.mv_max[1] = 4*( 16*( mb_height - mb_y - 1 ) + 24 );
+			if( h->param.b_no_mv_inter_slices == 0 )
+			{
+				h->mb.mv_min[1] = 4*( -16*mb_y - 24 );
+				h->mb.mv_max[1] = 4*( 16*( mb_height - mb_y - 1 ) + 24 );
+			}
+			else
+			{
+				h->mb.mv_min[1] = ( i_first_mb_x == 0 ) ? 4*( -16*( mb_y - i_first_mb_y ) - 8 ) : 4*( -16*( mb_y - i_first_mb_y + 1 ) - 8 );
+				h->mb.mv_max[1] = ( i_last_mb_x == h->sps->i_mb_width - 1 ) ? 4*( 16*( last_mb_y - mb_y - 1 ) + 8 ) : 4*( 16*( last_mb_y - mb_y - 1 - 1 ) + 8 );
+			}
             h->mb.mv_min_spel[1] = x264_clip3( h->mb.mv_min[1], X264_MAX(4*(-512+i_spel_border), -i_fmv_range), i_fmv_range );
             h->mb.mv_max_spel[1] = CLIP_FMV( h->mb.mv_max[1] );
             h->mb.mv_max_spel[1] = X264_MIN( h->mb.mv_max_spel[1], thread_mvy_range*4 );
@@ -338,23 +362,30 @@
 /* Max = 4 */
 static void predict_16x16_mode_available( unsigned int i_neighbour, int *mode, int *pi_count )
 {
-    if( i_neighbour & MB_TOPLEFT )
+    int b_l = i_neighbour & MB_LEFT;
+    int b_t = i_neighbour & MB_TOP;
+    if( b_l && b_t )
     {
         /* top and left available */
         *mode++ = I_PRED_16x16_V;
         *mode++ = I_PRED_16x16_H;
         *mode++ = I_PRED_16x16_DC;
-        *mode++ = I_PRED_16x16_P;
-        *pi_count = 4;
+        *pi_count = 3;
+        if( i_neighbour & MB_TOPLEFT )
+        {
+            /* top left available*/
+	    *mode++ = I_PRED_16x16_P;
+	    *pi_count = 4;
+        }
     }
-    else if( i_neighbour & MB_LEFT )
+    else if( b_l )
     {
         /* left available*/
         *mode++ = I_PRED_16x16_DC_LEFT;
         *mode++ = I_PRED_16x16_H;
         *pi_count = 2;
     }
-    else if( i_neighbour & MB_TOP )
+    else if( b_t )
     {
         /* top available*/
         *mode++ = I_PRED_16x16_DC_TOP;
@@ -372,23 +403,30 @@
 /* Max = 4 */
 static void predict_8x8chroma_mode_available( unsigned int i_neighbour, int *mode, int *pi_count )
 {
-    if( i_neighbour & MB_TOPLEFT )
+    int b_l = i_neighbour & MB_LEFT;
+    int b_t = i_neighbour & MB_TOP;
+    if( b_l && b_t )
     {
         /* top and left available */
         *mode++ = I_PRED_CHROMA_V;
         *mode++ = I_PRED_CHROMA_H;
         *mode++ = I_PRED_CHROMA_DC;
-        *mode++ = I_PRED_CHROMA_P;
-        *pi_count = 4;
+        *pi_count = 3;
+        if( i_neighbour & MB_TOPLEFT )
+        {
+            /* top left available */
+	    *mode++ = I_PRED_CHROMA_P;
+	    *pi_count = 4;
+        }
     }
-    else if( i_neighbour & MB_LEFT )
+    else if( b_l )
     {
         /* left available*/
         *mode++ = I_PRED_CHROMA_DC_LEFT;
         *mode++ = I_PRED_CHROMA_H;
         *pi_count = 2;
     }
-    else if( i_neighbour & MB_TOP )
+    else if( b_t )
     {
         /* top available*/
         *mode++ = I_PRED_CHROMA_DC_TOP;
Index: x264.c
===================================================================
--- x264.c	(revision 680)
+++ x264.c	(working copy)
@@ -170,6 +170,9 @@
     H1( "      --no-deblock            Disable loop filter\n" );
     H0( "  -f, --deblock <alpha:beta>  Loop filter AlphaC0 and Beta parameters [%d:%d]\n",
                                        defaults->i_deblocking_filter_alphac0, defaults->i_deblocking_filter_beta );
+    H0( "      --max-slicesize <integer>  Limit the size of each slice (in bytes) (0 => unlimited)\n");
+    H0( "      --mb-slicesize <integer>  Limit the size of each slice (in MacroBlocks) (0 => unlimited)\n");
+    H0( "      --no-mv-inter-slices    Prohibits any mv prediction between slices\n");
     H0( "      --interlaced            Enable pure-interlaced mode\n" );
     H0( "\n" );
     H0( "Ratecontrol:\n" );
@@ -355,6 +358,9 @@
 #define OPT_PROGRESS 261
 #define OPT_VISUALIZE 262
 #define OPT_LONGHELP 263
+#define OPT_SLICE_MAX 264
+#define OPT_SLICE_SIZE_MB 265
+#define OPT_NO_MV_INTER_SLICES 266
 
         static struct option long_options[] =
         {
@@ -425,6 +431,9 @@
             { "zones",   required_argument, NULL, 0 },
             { "qpfile",  required_argument, NULL, OPT_QPFILE },
             { "threads", required_argument, NULL, 0 },
+            { "max-slicesize", required_argument, NULL, OPT_SLICE_MAX },
+            { "mb-slicesize", required_argument, NULL, OPT_SLICE_SIZE_MB },
+            { "no-mv-inter-slices", no_argument, NULL, OPT_NO_MV_INTER_SLICES },
             { "thread-input", no_argument,  NULL, OPT_THREAD_INPUT },
             { "non-deterministic", no_argument, NULL, 0 },
             { "no-psnr", no_argument,       NULL, 0 },
@@ -527,6 +536,15 @@
                 param->i_scenecut_threshold = -1;
                 param->b_bframe_adaptive = 0;
                 break;
+            case OPT_SLICE_MAX:
+                param->slice_size_threshold = atoi(optarg);
+                break;
+            case OPT_SLICE_SIZE_MB:
+                param->i_slice_size_mb_threshold = atoi(optarg);
+                break;
+            case OPT_NO_MV_INTER_SLICES:
+                param->b_no_mv_inter_slices = 1;
+                break;
             case OPT_THREAD_INPUT:
                 b_thread_input = 1;
                 break;
Index: common/common.c
===================================================================
--- common/common.c	(revision 680)
+++ common/common.c	(working copy)
@@ -61,6 +61,9 @@
     param->i_fps_num       = 25;
     param->i_fps_den       = 1;
     param->i_level_idc     = 51; /* as close to "unrestricted" as we can get */
+    param->slice_size_threshold = 0;  /* by default: slice size threshold is infinity */
+    param->i_slice_size_mb_threshold = 0;  /* by default: slice size threshold is infinity */
+    param->b_no_mv_inter_slices = 0;  /* by default: mv prediction is allowed over the whole frame */
 
     /* Encoder parameters */
     param->i_frame_reference = 1;
Index: common/common.h
===================================================================
--- common/common.h	(revision 680)
+++ common/common.h	(working copy)
@@ -52,8 +52,7 @@
 
 #define X264_BFRAME_MAX 16
 #define X264_THREAD_MAX 128
-#define X264_SLICE_MAX 4
-#define X264_NAL_MAX (4 + X264_SLICE_MAX)
+#define X264_INIT_NAL_NUM 8 
 
 // number of pixels (per thread) in progress at any given time.
 // 16 for the macroblock in progress + 3 for deblocking + 3 for motion compensation filter + 2 for extra safety
@@ -244,7 +243,8 @@
     struct
     {
         int         i_nal;
-        x264_nal_t  nal[X264_NAL_MAX];
+        int         i_nals_allocated;
+        x264_nal_t  *nal;
         int         i_bitstream;    /* size of p_bitstream */
         uint8_t     *p_bitstream;   /* will hold data for all nal */
         bs_t        bs;
Index: x264.h
===================================================================
--- x264.h	(revision 680)
+++ x264.h	(working copy)
@@ -275,6 +275,12 @@
     int b_aud;                  /* generate access unit delimiters */
     int b_repeat_headers;       /* put SPS/PPS before each keyframe */
     int i_sps_id;               /* SPS and PPS id number */
+	
+	/* Slice related parameters */
+    int slice_size_threshold;   /* start new slice if current slice size exceeds this threshold in bytes. ( 0 is interpreted as infinity)   
+                                 * Note: provides no guarantee for maximum size of slice. Worst case: threshold + maximum bytes possible to encode one mb */
+	int i_slice_size_mb_threshold; /* start new slice if current slice size exceeds this threshold in #MB. ( 0 is interpreted as infinity) */
+	int b_no_mv_inter_slices; /* restrict mv predictions to MB in the same slice */
 } x264_param_t;
 
 typedef struct {
@@ -375,6 +381,7 @@
 {
     int i_ref_idc;  /* nal_priority_e */
     int i_type;     /* nal_unit_type_e */
+    int i_numb_mbs; /* number of macroblocks in this NAL */
 
     /* This data are raw payload */
     int     i_payload;
